/*
 * This file is part of TechReborn, licensed under the MIT License (MIT).
 *
 * Copyright (c) 2020 TechReborn
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
 * in the Software without restriction, including without limitation the rights
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in all
 * copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */

package techreborn.blockentity.generator;

import reborncore.api.IToolDrop;
import reborncore.client.containerBuilder.IContainerProvider;
import reborncore.client.containerBuilder.builder.BuiltContainer;
import reborncore.client.containerBuilder.builder.ContainerBuilder;
import reborncore.common.blocks.BlockMachineBase;
import reborncore.common.powerSystem.PowerAcceptorBlockEntity;
import reborncore.common.powerSystem.PowerSystem;
import reborncore.common.util.StringUtils;
import team.reborn.energy.EnergyTier;
import techreborn.blocks.generator.BlockSolarPanel;
import techreborn.init.TRBlockEntities;
import techreborn.init.TRContent;
import techreborn.init.TRContent.SolarPanels;

import java.util.List;
import net.minecraft.class_124;
import net.minecraft.class_1657;
import net.minecraft.class_1799;
import net.minecraft.class_2248;
import net.minecraft.class_2350;
import net.minecraft.class_2487;
import net.minecraft.class_2561;
import net.minecraft.class_2585;

public class SolarPanelBlockEntity extends PowerAcceptorBlockEntity implements IToolDrop, IContainerProvider {


	//	State ZEROGEN: No exposure to sun
	//	State NIGHTGEN: Has direct exposure to sun
	//	State DAYGEN: Has exposure to sun and weather is sunny and not raining/thundering
	public static final int ZEROGEN = 0;
	public static final int NIGHTGEN = 1;
	public static final int DAYGEN = 2;

	private int state = ZEROGEN;
	private int prevState = ZEROGEN;

	private SolarPanels panel;

	public SolarPanelBlockEntity() {
		super(TRBlockEntities.SOLAR_PANEL);
	}

	public SolarPanelBlockEntity(SolarPanels panel) {
		super(TRBlockEntities.SOLAR_PANEL);
		this.panel = panel;
	}

	private void updatePanel() {
		if (field_11863 == null) {
			return;
		}
		class_2248 panelBlock = field_11863.method_8320(field_11867).method_11614();
		if (panelBlock instanceof BlockSolarPanel) {
			BlockSolarPanel solarPanelBlock = (BlockSolarPanel) panelBlock;
			panel = solarPanelBlock.panelType;
		}
	}


	// Setters and getters for the GUI to sync
	private void setSunState(int state) {
		this.state = state;
	}

	public int getSunState() {
		return state;
	}

	SolarPanels getPanel() {
		if (panel == null) {
			updatePanel();
		}
		return panel;
	}

	private void updateState() {
		if (field_11863.method_8311(field_11867.method_10084())) {
			this.setSunState(NIGHTGEN);

			if (!field_11863.method_8419() && !field_11863.method_8546() && field_11863.method_8530()) {
				this.setSunState(DAYGEN);
			}
		} else {
			this.setSunState(ZEROGEN);
		}

		if (prevState != this.getSunState()) {
			boolean isGenerating = getSunState() == DAYGEN;

			field_11863.method_8501(field_11867, field_11863.method_8320(field_11867).method_11657(BlockMachineBase.ACTIVE, isGenerating));

			prevState = this.getSunState();
		}
	}

	public int getGenerationRate() {
		int rate = 0;

		switch (getSunState()) {
			case DAYGEN:
				rate = getPanel().generationRateD;
				break;
			case NIGHTGEN:
				rate = getPanel().generationRateN;
		}

		return rate;
	}


	// Overrides

	@Override
	public void method_16896() {
		super.method_16896();

		if (field_11863.field_9236) {
			return;
		}

		if (getPanel() == TRContent.SolarPanels.CREATIVE) {
			checkOverfill = false;
			setEnergy(Integer.MAX_VALUE);
			return;
		}

		// State checking and updating
		if (field_11863.method_8510() % 20 == 0) {
			updateState();
		}

		// Power generation calculations
		addEnergy(getGenerationRate());
	}

	@Override
	public double getBaseMaxPower() {
		return getPanel().internalCapacity;
	}

	@Override
	public boolean canAcceptEnergy(final class_2350 direction) {
		return false;
	}

	@Override
	public boolean canProvideEnergy(final class_2350 direction) {
		return true;
	}

	@Override
	public double getBaseMaxOutput() {
		// Solar panel output will only be limited by the cables the users use
		return EnergyTier.EXTREME.getMaxOutput();
	}

	@Override
	public double getBaseMaxInput() {
		return 0;
	}

	@Override
	public boolean canBeUpgraded() {
		return false;
	}

	@Override
	public boolean hasSlotConfig() {
		return false;
	}

	@Override
	public EnergyTier getTier() {
		return getPanel().powerTier;
	}

	@Override
	public void checkTier() {
		// Nope
	}

	@Override
	public void addInfo(List<class_2561> info, boolean isReal, boolean hasData) {
		if (panel == SolarPanels.CREATIVE) {
			return;
		}
		info.add(new class_2585(class_124.field_1080 + StringUtils.t("reborncore.tooltip.energy.maxEnergy") + ": "
				+ class_124.field_1065 + PowerSystem.getLocaliszedPowerFormatted(getMaxPower())));

		info.add(new class_2585(class_124.field_1080 + StringUtils.t("techreborn.tooltip.generationRate.day") + ": "
				+ class_124.field_1065 + PowerSystem.getLocaliszedPowerFormatted(panel.generationRateD)));

		info.add(new class_2585(class_124.field_1080 + StringUtils.t("techreborn.tooltip.generationRate.night") + ": "
				+ class_124.field_1065 + PowerSystem.getLocaliszedPowerFormatted(panel.generationRateN)));

		info.add(new class_2585(class_124.field_1080 + StringUtils.t("reborncore.tooltip.energy.tier") + ": "
				+ class_124.field_1065 + StringUtils.toFirstCapitalAllLowercase(getTier().toString())));
	}

	@Override
	public void method_11014(class_2487 tag) {
		if (field_11863 == null) {
			// We are in BlockEntity.create method during chunk load.
			this.checkOverfill = false;
		}
		updatePanel();
		super.method_11014(tag);
	}

	// MachineBaseBlockEntity
	@Override
	public void onLoad() {
		super.onLoad();
		updatePanel();
	}

	// IToolDrop
	@Override
	public class_1799 getToolDrop(final class_1657 playerIn) {
		return new class_1799(getBlockType());
	}

	@Override
	public BuiltContainer createContainer(int syncID, final class_1657 player) {
		return new ContainerBuilder("solar_panel").player(player.field_7514).inventory().hotbar().addInventory()
				.blockEntity(this).syncEnergyValue()
				.sync(this::getSunState, this::setSunState)
				.addInventory().create(this, syncID);
	}
}