/*
 * This file is part of TechReborn, licensed under the MIT License (MIT).
 *
 * Copyright (c) 2020 TechReborn
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
 * in the Software without restriction, including without limitation the rights
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in all
 * copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */

package techreborn.blockentity.machine.iron;

import net.minecraft.class_1657;
import net.minecraft.class_1799;
import net.minecraft.class_2248;
import net.minecraft.class_2487;
import net.minecraft.class_2591;
import net.minecraft.class_2609;
import net.minecraft.class_2680;
import reborncore.api.IToolDrop;
import reborncore.api.blockentity.InventoryProvider;
import reborncore.common.blockentity.MachineBaseBlockEntity;
import reborncore.common.blockentity.SlotConfiguration;
import reborncore.common.blocks.BlockMachineBase;
import reborncore.common.util.RebornInventory;
import techreborn.config.TechRebornConfig;

public abstract class AbstractIronMachineBlockEntity extends MachineBaseBlockEntity implements InventoryProvider, IToolDrop, SlotConfiguration.SlotFilter {

	public RebornInventory<?> inventory;
	public int burnTime;
	public int totalBurnTime;
	public int progress;
	public int totalCookingTime;
	int fuelSlot;
	class_2248 toolDrop;
	boolean active = false;

	public AbstractIronMachineBlockEntity(class_2591<?> blockEntityTypeIn, int fuelSlot, class_2248 toolDrop) {
		super(blockEntityTypeIn);
		this.fuelSlot = fuelSlot;
		// default value for vanilla smelting recipes is 200
		this.totalCookingTime = (int) (200 / TechRebornConfig.cookingScale);
		this.toolDrop = toolDrop;
	}
	
	/**
	 * Checks that we have all inputs and can put output into slot
	 * @return
	 */
	protected abstract boolean canSmelt();

	/**
	 * Turn ingredients into the appropriate smelted
	 * item in the output slot
	 */
	protected abstract void smelt();

	/**
	 * Returns the number of ticks that the supplied fuel item will keep the
	 * furnace burning, or 0 if the item isn't fuel
	 * @param stack Itemstack of fuel
	 * @return Integer Number of ticks
	 */
	private int getItemBurnTime(class_1799 stack) {
		if (stack.method_7960()) {
			return 0;
		}
		return (int) (class_2609.method_11196().getOrDefault(stack.method_7909(), 0) * TechRebornConfig.fuelScale);
	}
	
	/**
	 * Returns remaining fraction of fuel burn time 
	 * @param scale Scale to use for burn time
	 * @return int scaled remaining fuel burn time 
	 */
	public int getBurnTimeRemainingScaled(int scale) {
		if (totalBurnTime == 0) {
			return 0;
		}

		return burnTime * scale / totalBurnTime;
	}
	
	/**
	 * Returns crafting progress 
	 * @param scale Scale to use for crafting progress
	 * @return int Scaled crafting progress 
	 */
	public int getProgressScaled(int scale) {
		if (totalCookingTime > 0) {
			return progress * scale / totalCookingTime;
		}
		return 0;
	}

	/**
	 * Returns true if Iron Machine is burning fuel thus can do work
	 * @return Boolean True if machine is burning
	 */
	public boolean isBurning() {
		return burnTime > 0;
	}

	private void updateState() {
		class_2680 state = field_11863.method_8320(field_11867);
		if (state.method_11614() instanceof BlockMachineBase) {
			BlockMachineBase blockMachineBase = (BlockMachineBase) state.method_11614();
			if (state.method_11654(BlockMachineBase.ACTIVE) != burnTime > 0)
				blockMachineBase.setActive(burnTime > 0, field_11863, field_11867);
		}
	}

	// MachineBaseBlockEntity
	@Override
	public void method_11014(class_2487 compoundTag) {
		super.method_11014(compoundTag);
		burnTime = compoundTag.method_10550("BurnTime");
		totalBurnTime = compoundTag.method_10550("TotalBurnTime");
		progress = compoundTag.method_10550("Progress");
	}

	@Override
	public class_2487 method_11007(class_2487 compoundTag) {
		 super.method_11007(compoundTag);
		 compoundTag.method_10569("BurnTime", burnTime);
		 compoundTag.method_10569("TotalBurnTime", totalBurnTime);
		 compoundTag.method_10569("Progress", progress);
		 return compoundTag;
	}

	@Override
	public void method_16896() {
		super.method_16896();
		if(field_11863.field_9236){
			return;
		}
		boolean isBurning = isBurning();
		if (isBurning) {
			--burnTime;
		}
		
		if (!isBurning && canSmelt()) {
			burnTime = totalBurnTime = getItemBurnTime(inventory.method_5438(fuelSlot));
			if (burnTime > 0) {
				// Fuel slot
				class_1799 fuelStack = inventory.method_5438(fuelSlot);
				if (fuelStack.method_7909().method_7857()) {
					inventory.method_5447(fuelSlot, new class_1799(fuelStack.method_7909().method_7858()));
				} else if (fuelStack.method_7947() > 1) {
					inventory.shrinkSlot(fuelSlot, 1);
				} else if (fuelStack.method_7947() == 1) {
					inventory.method_5447(fuelSlot, class_1799.field_8037);
				}
			}
		}
		
		if (isBurning() && canSmelt()) {
			++progress;
			if (progress == totalCookingTime) {
				progress = 0;
				smelt();
			}
		} else if(!canSmelt()) {
			progress = 0;
		}
		
		if (isBurning != isBurning()) {
			inventory.setChanged();
			updateState();
		}
		if (inventory.hasChanged()) {
			method_5431();
		}
	}

	@Override
	public boolean canBeUpgraded() {
		return false;
	}

	// InventoryProvider
	@Override
	public RebornInventory<?> getInventory() {
		return inventory;
	}
	
	// IToolDrop
	@Override
	public class_1799 getToolDrop(class_1657 entityPlayer) {
		return new class_1799(toolDrop);
	}

	public int getBurnTime() {
		return this.burnTime;
	}

	public void setBurnTime(int burnTime) {
		this.burnTime = burnTime;
	}

	public int getTotalBurnTime() {
		return this.totalBurnTime;
	}

	public void setTotalBurnTime(int totalBurnTime) {
		this.totalBurnTime = totalBurnTime;
	}
	
	public int getProgress() {
		return progress;
	}

	public void setProgress(int progress) {
		this.progress = progress;
	}


}